
Player = Core.class(Sprite)

function Player:init()
  
  PLAYER = self -- Make instance globally available
  
  self.position = {x = 0, y = 0}
  self.velocity = {x = 0, y = 0}

  -- Load

  local pack = TexturePack.new("images/player.txt", "images/player.png")
  
  local frames = {}
  
  table.insert(frames, Bitmap.new(pack:getTextureRegion("idle1.png"))) --1
  table.insert(frames, Bitmap.new(pack:getTextureRegion("jump1.png"))) --2
  table.insert(frames, Bitmap.new(pack:getTextureRegion("fall1.png"))) -- 3
  table.insert(frames, Bitmap.new(pack:getTextureRegion("slide1.png"))) -- 4
  table.insert(frames, Bitmap.new(pack:getTextureRegion("dead1.png"))) -- 5
  table.insert(frames, Bitmap.new(pack:getTextureRegion("run1.png"))) -- 6
  table.insert(frames, Bitmap.new(pack:getTextureRegion("run2.png")))
  table.insert(frames, Bitmap.new(pack:getTextureRegion("run3.png")))
  table.insert(frames, Bitmap.new(pack:getTextureRegion("run4.png")))
  table.insert(frames, Bitmap.new(pack:getTextureRegion("run5.png")))
  table.insert(frames, Bitmap.new(pack:getTextureRegion("run6.png")))
  table.insert(frames, Bitmap.new(pack:getTextureRegion("run7.png")))
  table.insert(frames, Bitmap.new(pack:getTextureRegion("run8.png")))
  
  local clip = MovieClip.new{
    {1, 1, frames[1]}, -- idle
    
    {2, 2, frames[2]}, -- jump
    
    {3, 3, frames[3]}, -- fall
    
    {4, 4, frames[4]}, -- slide
    
    {5, 5, frames[5]}, -- dead
    
    {06, 09, frames[6]}, -- run
    {09, 12, frames[7]},
    {12, 16, frames[8]},
    {16, 20, frames[9]},
    {20, 24, frames[10]},
    {24, 28, frames[11]},
    {28, 32, frames[12]},
    {32, 36, frames[13]},
  }
  
  self:addChild(clip)
  
  clip:setAnchorPosition(32, 0, 0) -- Because MovieClip can't do setAnchorPoint() grr
  
  self.clip = clip
  
  self:reset()
end

function Player:animate() -- Set players animation
  
  local clip = self.clip -- Our MovieClip
  
  local anim = self.state
  
  if anim == "running" then
    clip:setGotoAction(36, 6) -- Looping animation
    clip:gotoAndPlay(6)
  
  elseif anim == "idle" then
    clip:gotoAndStop(1) -- The rest are one frame animations
    
  elseif anim == "jumping" then
    clip:gotoAndStop(2)
    
  elseif anim == "falling" then
    clip:gotoAndStop(3)
  
  elseif anim == "slide" then
    clip:gotoAndStop(4)
  
  elseif anim == "dead" then
    clip:gotoAndStop(5)
  
  end
end

function Player:setState(state) -- Set players new state
  if self.state ~= state then
    self.state = state
    self:animate()
  end
end

function Player:reset()
  local p, v = self.position, self.velocity -- reset position and velocity
  local x, y = 64, 0 -- Initial position in world
  p.x, p.y = x, y
  v.x, v.y = 0, 0
  
  self.onGround = false
  self.isDead = false
  
  self:setState("falling")

  self:setPosition(x, y)
end

function Player:touchedGround() -- Check if on ground and return a corrected y value if true
  local game = GAME
  local p = self.position
  local x, y = p.x, p.y + 65 -- Get collision position
  
  local groundObjects = game.groundObjects
  for i = 1, #groundObjects do
    local object = groundObjects[i]
    local ox, oy = object:getPosition() -- Get objects position
    local ow = object:getWidth()
    
    if x > ox and x < ox + ow then -- is it in x range?
      if y >= oy then -- is it inside the object?
        return true, oy - 64 -- return corrected y position
      end
    end
  end
  return false, 0 -- No collisions happened
end

function Player:checkWalls() -- Check for collisions with walls and kill player if required
 
 local function rectIntersectsRect(r1, r2) -- Return true if rectangles intersect (overlap)
    if r1.x + r1.w >= r2.x and r1.x <= r2.x + r2.w and r1.y + r1.h >= r2.y and r1.y <= r2.y + r2.h then
      return true
    end
    return false
  end
 
  local game = GAME
  
  local p, v = self.position, self.velocity
  local playerRect = {
    x = p.x - 16,
    y = p.y + 8,
    w = 32,
    h = 56,
  }
  
  local wallObjects = game.wallObjects
  for i = 1, #wallObjects do
    local object = wallObjects[i]
    local wallRect = {
      x = object:getX(),
      y = object:getY(),
      w = object:getWidth(),
      h = object:getHeight(),
    }
    
    if rectIntersectsRect(playerRect, wallRect) then -- check if there was a collision
      if v.x > 0 then
        v.x = -64
      else
        v.x = -64
      end
      v.y = -768
      self:setState("dead")
      self.dead = true
      return true -- Exit here because there is no need to check others since a collision already happened
    end
  end
  return false
end

function Player:update(dt) -- Call this ever frame to update the player
  local game = GAME
  
  local p, v, g = self.position, self.velocity, game.gravity
  
  v.x = v.x + g.x -- Apply gravity
  v.y = v.y + g.y
  
  local x = p.x + (v.x * dt) -- Determine desired position
  local y = p.y + (v.y * dt)
  
  self.onGround = false
  
  if v.y > 0 then -- we must be falling if velocity is positive
    local grounded, actualY = self:touchedGround() -- check if on ground
    if grounded then
      y = actualY -- Correct y if we collided with a ground object
      self.onGround = true
      v.y = 0
      
    else
      if self.state ~= "dead" then
        self:setState("falling") -- Set as falling
      end
      self.onGround = false
      
      if y > HEIGHT - 64 then -- If we fell past this point we died
        self:setState("dead")
        self.isDead = true
        v.y = -1024
      end
    end
  end
  
  self:checkWalls()

  if v.x == 0 then
    if self.onGround then
      self:setState("idle")
    end
  else
    if self.onGround then
      self:setState("running")
    end
  end

  if x < 0 then -- Stop player going out of the display
    x = 0
  elseif x > WIDTH then
    x = WIDTH
  end

  p.x, p.y = x, y -- Update player actual position in world and on display
  self:setPosition(x, y)
end
